
;----------------------------------------------------------------------
; Reads a file containing a console screen format into an array
; of structure CHAR_INFO totalling 8000 bytes, and INVOKE the API
; to display it all at once: WriteConsoleOutput(). Wait a key and
; terminates the program.
;----------------------------------------------------------------------
; Input file: ASCII.scn
;----------------------------------------------------------------------
; Author: frktons @ MASM32 forum
; Date: 02/sep/2010. 
;---------------------------------------------------------------------- 
; The challenge is to get rid of the external file and replace it with
; a PROC that fill the ConsoleScreen area with the bytes needed to re-
; create the same screen in less than 600-700 bytes.
; Enjoy the challenge. Frank
;----------------------------------------------------------------------


include \masm32\include\masm32rt.inc

GetConsole PROTO :DWORD
DisplayFmt PROTO :DWORD

;----------------------------------------------------------------------

BufRows EQU 25                 ;buffer height (rows)
BufCols EQU 80                 ;buffer width (columns)
BufLen  EQU BufRows*BufCols*4  ;string length (tchars)

;----------------------------------------------------------------------


.data


    ConTitle  db "  ----------     **  Extended ASCII chart displayable characters  **      ----------", 0   
    inpbyte   DWORD  1
    FmtName   db "ASCII.scn",0  
          
         

.data?

    ConsoleScreen CHAR_INFO 2000 dup (<>)
   

    hFile     HANDLE ? 
    cci       CONSOLE_CURSOR_INFO <>        


    wHnd      HANDLE ? 
    rHnd      HANDLE ?

    windowSize SMALL_RECT <>
    bufferSize COORD      <>
    startPoint COORD      <>

    howmany dd ?
    buffer  INPUT_RECORD <>


.code

start: 

Main PROC

    CALL   InitProc

    CALL   ConsoleSize

    INVOKE GetConsole, offset FmtName 

    INVOKE DisplayFmt, offset ConsoleScreen
       
    CALL   AnyKey

    CALL   ShowTheCursor

finish: INVOKE ExitProcess,0

        ret

Main ENDP

; -------------------------------------------------------------------------   

ConsoleSize PROC

    INVOKE SetConsoleTitle, ADDR ConTitle
  
    CALL   HideTheCursor

    INVOKE SetConsoleWindowInfo, wHnd, TRUE, ADDR windowSize

    INVOKE SetConsoleScreenBufferSize, wHnd, DWORD PTR bufferSize

    ret

ConsoleSize ENDP

; -------------------------------------------------------------------------

GetConsole PROC szFmtName:DWORD 

    mov hFile, fopen(szFmtName)

    mov ebx, fread(hFile, ADDR ConsoleScreen, BufLen)
    
    fclose hFile

    ret 
    
GetConsole ENDP

; -------------------------------------------------------------------------

ShowTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,TRUE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
ShowTheCursor ENDP

; -------------------------------------------------------------------------
    


HideTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,FALSE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
HideTheCursor ENDP

; -------------------------------------------------------------------------

InitProc PROC

    INVOKE GetStdHandle, STD_INPUT_HANDLE
    mov rHnd,eax

    INVOKE GetStdHandle, STD_OUTPUT_HANDLE
    mov wHnd,eax  

    mov windowSize.Left, 0
    mov windowSize.Right, 79
    mov windowSize.Top, 0
    mov windowSize.Bottom, 24
   

    mov bufferSize.x, 80
    mov bufferSize.y, 25

    mov startPoint.x, 0
    mov startPoint.y, 0


    ret

InitProc ENDP

; -------------------------------------------------------------------------

DisplayFmt PROC szFmtName:DWORD 

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

DisplayFmt ENDP

; -------------------------------------------------------------------------
;Returns: key code in buffer.KeyEvent.wVirtualKeyCode WORD size 
; -------------------------------------------------------------------------

AnyKey PROC

again:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz again

    cmp buffer.KeyEvent.bKeyDown,0
    jz again

    ret 

AnyKey ENDP

; -------------------------------------------------------------------------

end start