
;----------------------------------------------------------------------
; Fills a 8000 bytes console screen buffer with the same data
; of the file ascii.scn, generating the data, and INVOKE the API
; WriteConsoleOutput() to display it all at once, Wait a key and
; terminates the program.
;----------------------------------------------------------------------
; Input file: no input file in this version
; The content of ascii.scn is generated inside the program
; by the proc BuildConsole and other called procs.
;----------------------------------------------------------------------
; Author: frktons @ MASM32 forum
; Date: 13/sep/2010.  Version 0.3
;---------------------------------------------------------------------- 
; The challenge is to get rid of the external file and replace it with
; PROCs that fills the ConsoleScreen area with the bytes needed to re-
; create the same screen in less than 600-700 bytes.
; Enjoy the challenge. Frank
;----------------------------------------------------------------------


include \masm32\include\masm32rt.inc


DisplayFmt PROTO :DWORD
BuildBox   PROTO :DWORD, :DWORD

;----------------------------------------------------------------------

BufRows EQU 25                 ;buffer height (rows)
BufCols EQU 80                 ;buffer width (columns)
BufLen  EQU BufRows*BufCols*4  ;string length (tchars)



;----------------------------------------------------------------------
; Symbolic constants used in the program
;----------------------------------------------------------------------

LenScreenTitle = 47            ;Length of the Screen Title string 
EnhancedColor  = 5 * 16 + 15   ;Enhanced color = Bright White on Magenta
                               ;White = 15 / Magenta = 5 

TitleCol = 17
TitleStartPoint= (80 * 4) + (TitleCol - 1) * 4   

Four = 4  
LenBoxLine = 78    

Zero = '0'
One  = '1'
Nine = '9'
Space = ' '
MaxAsciiCode = 255                        

;----------------------------------------------------------------------


.data


ConTitle  db " -----------   **  Extended ASCII chart displayable characters  **  ----------", 0
PtrConTitle DWORD ConTitle + 17  ; Points to "  Extended..." part of ConTitle  
  
inpbyte   DWORD  1
    
;----------------------------------------------------------------------
; ASCII chars to draw a simple box
;----------------------------------------------------------------------    

    TopLeft      db   218
    TopRight     db   191
    BottomLeft   db   192
    BottomRight  db   217
    HorizLine    db   196
    VerticLine   db   179  

;----------------------------------------------------------------------

    Unit     BYTE One
    Ten      BYTE Space
    Hundred  BYTE Space
    EndOfTask  BYTE 0
    

    AsciiCode  DWORD 205F2000H  ; 4 bytes = 1 CHAR_INFO
                                ; char 1 White on Magenta Attribute       


;----------------------------------------------------------------------   
         

.data?

    ConsoleScreen CHAR_INFO 2000 dup (<>)
    
   
    cci       CONSOLE_CURSOR_INFO <>        

    wHnd      HANDLE ? 
    rHnd      HANDLE ?

    windowSize SMALL_RECT <>
    bufferSize COORD      <>
    startPoint COORD      <>

    howmany dd ?
    buffer  INPUT_RECORD <>

    PtrBuf   DWORD ? 

    PtrBuf2  DWORD ? 




.code

start: 

Main PROC

    CALL   InitProc

    CALL   ConsoleSize

    CALL   BuildConsole 

    INVOKE DisplayFmt, offset ConsoleScreen
       
    CALL   AnyKey

    CALL   ShowTheCursor

finish: 

    INVOKE ExitProcess,0

    ret

Main ENDP

; -------------------------------------------------------------------------   

ConsoleSize PROC

    INVOKE SetConsoleTitle, ADDR ConTitle
  
    CALL   HideTheCursor

    INVOKE SetConsoleWindowInfo, wHnd, TRUE, ADDR windowSize

    INVOKE SetConsoleScreenBufferSize, wHnd, DWORD PTR bufferSize

    ret

ConsoleSize ENDP

; -------------------------------------------------------------------------
; The buffer is filled before displaying it.
; This PROC and the CALLed ones replace the external file ASCII.scn
;--------------------------------------------------------------------------

BuildConsole PROC  

    CALL SetBufferColor

    INVOKE BuildBox, 1, 3
    INVOKE BuildBox, Four, 25

    CALL   WriteScreenTitle
    CALL   DisplaySequence

    ret 
    
BuildConsole ENDP

; -------------------------------------------------------------------------
; Fill the buffer with a box of 80 characters starting at line requested
; till the number of line passed as II parameter
;--------------------------------------------------------------------------

BuildBox PROC Row1:DWORD, Row2:DWORD

 
    mov  eax,  Row1
    sub  eax,  1
    imul eax,  320
    mov  edx,  eax

    lea  eax,  ConsoleScreen
    add  eax,  edx


    movzx ebx, byte ptr [TopLeft]
    mov   byte ptr [eax],bl

    CALL  BoxLine


    movzx ebx, byte ptr [TopRight]

    mov  byte ptr [eax], bl 
    add  eax, Four

    mov  ecx, Row2
    sub  ecx, Row1
    dec  ecx
    movzx ebx, byte ptr [VerticLine]

FillBorder:


    mov  byte ptr [eax], bl 

    add  eax, Four
    mov  edx, Four
    imul edx, LenBoxLine

    add  eax, edx
    mov  byte ptr [eax], bl 
    add  eax, Four    
    dec  ecx
    jnz  FillBorder

    movzx ebx,byte ptr [BottomLeft]
    mov   byte ptr [eax], bl 

    CALL  BoxLine

    movzx ebx, byte ptr [BottomRight]
    mov   byte ptr [eax], bl 


    ret


BuildBox ENDP

; -------------------------------------------------------------------------
; Writes the horizontal line of the Box
;--------------------------------------------------------------------------

BoxLine PROC

    add   eax, Four

    mov   ecx, LenBoxLine

    movzx ebx, byte ptr [HorizLine]

FillBoxLine:

    mov   byte ptr [eax], bl
    add   eax, Four
    LOOP  FillBoxLine

    ret 
    
BoxLine ENDP

; -------------------------------------------------------------------------
; Writes the sequence of ASCII codes and characters
;--------------------------------------------------------------------------

DisplaySequence PROC

    mov   eax, offset ConsoleScreen
    mov   PtrBuf,  eax 

    add   eax, ( Four * 80 * Four ) + 20

    mov   PtrBuf2, eax  

    lea   ebx, offset AsciiCode
    mov   eax, [ebx]
    mov   esi, 1    ; Counts the columns to write, 13 


PrintCol:

    inc   eax
    mov   ebx,   PtrBuf2 
    mov   [ebx], eax
    cmp   al, 255
    jne   CarryOn
    mov   EndOfTask, 1

CarryOn:

    mov   edx, PtrBuf2
    sub   edx, 8
    mov   cl,  Unit
    mov   BYTE PTR [edx], cl

    sub   edx, Four
    mov   cl,  Ten
    mov   BYTE PTR [edx], cl

    sub   edx, Four
    mov   cl,  Hundred
    mov   BYTE PTR [edx], cl

    cmp   EndOfTask, 1
    je    EndCycle

    inc   Unit
    cmp   Unit, Nine
    jle   NextCol

    mov   Unit, Zero

    cmp   Ten,  Space
    jne   IncTen
    mov   Ten,  One
    jmp   NextCol

IncTen:

    inc   Ten
    cmp   Ten,  Nine
    jng   NextCol 

    mov   Ten,  Zero

HundredCheck:

    cmp   Hundred,  Space
    jne   IncHundred 
    mov   Hundred,  BYTE PTR One
    jmp   NextCol

IncHundred:

    inc   Hundred

NextCol:

    add   PtrBuf2,  24
    inc   esi
    cmp   esi,  13
    jg    NextRow
    jmp   PrintCol  

NextRow:

    mov   esi,  1
    add   PtrBuf2,  8    

    jmp   PrintCol
    
EndCycle:

    ret
    
DisplaySequence ENDP

; -------------------------------------------------------------------------
; Writes the Screen Title with enhanced color attribute White on Magenta
;--------------------------------------------------------------------------

WriteScreenTitle PROC

    lea  eax, ConsoleScreen
    add  eax, TitleStartPoint
    mov  esi, PtrConTitle

    mov  bl,  BYTE PTR EnhancedColor
    mov  ecx, LenScreenTitle

WriteTitle:

    mov  dl,  BYTE PTR [esi]

    mov  BYTE PTR [eax], dl

    add  eax, 2

    mov  BYTE PTR [eax], bl

    add  eax, 2

    inc  esi

    LOOP WriteTitle

    ret

WriteScreenTitle ENDP

; -------------------------------------------------------------------------
; The buffer is initialized with main color attribute Bright Yellow on Blue
;--------------------------------------------------------------------------

SetBufferColor PROC  

    push edi

    mov edi, offset ConsoleScreen
    mov eax, 201E2020H
    mov ecx, 2000
    rep stosd
    	
    pop edi

    ret 
    
SetBufferColor ENDP

; -------------------------------------------------------------------------

ShowTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,TRUE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
ShowTheCursor ENDP

; -------------------------------------------------------------------------
    


HideTheCursor PROC

    INVOKE GetConsoleCursorInfo,wHnd,addr cci
    mov cci.bVisible,FALSE
    INVOKE SetConsoleCursorInfo,wHnd,addr cci
    ret
    
HideTheCursor ENDP

; -------------------------------------------------------------------------

InitProc PROC

    INVOKE GetStdHandle, STD_INPUT_HANDLE
    mov rHnd,eax

    INVOKE GetStdHandle, STD_OUTPUT_HANDLE
    mov wHnd,eax  

    mov windowSize.Left, 0
    mov windowSize.Right, 79
    mov windowSize.Top, 0
    mov windowSize.Bottom, 24
   

    mov bufferSize.x, 80
    mov bufferSize.y, 25

    mov startPoint.x, 0
    mov startPoint.y, 0


    ret

InitProc ENDP

; -------------------------------------------------------------------------

DisplayFmt PROC szFmtName:DWORD 

    INVOKE WriteConsoleOutput, wHnd, szFmtName, DWORD PTR bufferSize, 
           DWORD PTR startPoint, offset windowSize 

    ret

DisplayFmt ENDP

; -------------------------------------------------------------------------
;Returns: key code in buffer.KeyEvent.wVirtualKeyCode WORD size 
; -------------------------------------------------------------------------

AnyKey PROC

again:  

    INVOKE ReadConsoleInput,rHnd,offset buffer,1,offset howmany
    cmp buffer.EventType,KEY_EVENT
    jnz again

    cmp buffer.KeyEvent.bKeyDown,0
    jz again

    ret 

AnyKey ENDP

; -------------------------------------------------------------------------

end start